@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2007, Jason Kivlighn (jkivlighn@gmail.com)
- * Copyright (C) 2007, Creative Commons (http://creativecommons.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2007, Jason Kivlighn <jkivlighn@gmail.com>
+ * Copyright (C) 2007, Creative Commons <http://creativecommons.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,8 +31,10 @@
 #include <libtracker-db/tracker-db-interface-sqlite.h>
 #include <libtracker-db/tracker-db-manager.h>
 
+#include "tracker-class.h"
 #include "tracker-data-manager.h"
 #include "tracker-data-query.h"
+#include "tracker-ontologies.h"
 #include "tracker-sparql-query.h"
 
 GPtrArray*
@@ -62,7 +64,16 @@ tracker_data_query_rdf_type (gint id)
 
 		ret = g_ptr_array_sized_new (20);
 		while (tracker_db_cursor_iter_next (cursor)) {
-			g_ptr_array_add (ret, g_strdup (tracker_db_cursor_get_string (cursor, 0)));
+			const gchar *class_uri;
+			TrackerClass *cl;
+
+			class_uri = tracker_db_cursor_get_string (cursor, 0);
+			cl = tracker_ontologies_get_class_by_uri (class_uri);
+			if (!cl) {
+				g_critical ("Unknown class %s", class_uri);
+				continue;
+			}
+			g_ptr_array_add (ret, cl);
 		}
 		g_object_unref (cursor);
 	}
