@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,6 +29,16 @@
 #include "tracker-utils.h"
 #include "tracker-marshal.h"
 
+/**
+ * SECTION:tracker-storage
+ * @short_description: Removable storage and mount point convenience API
+ * @include: libtracker-miner/tracker-miner.h
+ *
+ * This API is a convenience to to be able to keep track of volumes
+ * which are mounted and also the type of removable media available.
+ * The API is built upon the top of GIO's #GMount, #GDrive and #GVolume API.
+ **/
+
 #define TRACKER_STORAGE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_STORAGE, TrackerStoragePrivate))
 
 typedef struct {
@@ -42,6 +52,7 @@ typedef struct {
 	gchar *mount_point;
 	gchar *uuid;
 	guint removable : 1;
+	guint optical : 1;
 } MountInfo;
 
 typedef struct {
@@ -51,7 +62,8 @@ typedef struct {
 
 typedef struct {
 	GSList *roots;
-	gboolean only_removable;
+	TrackerStorageType type;
+	gboolean exact_match;
 } GetRoots;
 
 static void     tracker_storage_finalize (GObject        *object);
@@ -94,11 +106,13 @@ tracker_storage_class_init (TrackerStorageClass *klass)
 		              G_SIGNAL_RUN_LAST,
 		              0,
 		              NULL, NULL,
-		              tracker_marshal_VOID__STRING_STRING,
+		              tracker_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN,
 		              G_TYPE_NONE,
-		              2,
+		              4,
 		              G_TYPE_STRING,
-		              G_TYPE_STRING);
+		              G_TYPE_STRING,
+		              G_TYPE_BOOLEAN,
+		              G_TYPE_BOOLEAN);
 
 	signals[MOUNT_POINT_REMOVED] =
 		g_signal_new ("mount-point-removed",
@@ -253,6 +267,22 @@ mount_info_find (GNode       *root,
 	return (node) ? node->data : NULL;
 }
 
+static TrackerStorageType
+mount_info_get_type (MountInfo *info)
+{
+	TrackerStorageType mount_type = 0;
+
+	if (info->removable) {
+		mount_type |= TRACKER_STORAGE_REMOVABLE;
+	}
+
+	if (info->optical) {
+		mount_type |= TRACKER_STORAGE_OPTICAL;
+	}
+
+	return mount_type;
+}
+
 static gchar *
 mount_point_normalize (const gchar *mount_point)
 {
@@ -272,7 +302,8 @@ static GNode *
 mount_add_hierarchy (GNode       *root,
                      const gchar *uuid,
                      const gchar *mount_point,
-                     gboolean     removable)
+                     gboolean     removable,
+                     gboolean     optical)
 {
 	MountInfo *info;
 	GNode *node;
@@ -289,6 +320,7 @@ mount_add_hierarchy (GNode       *root,
 	info->mount_point = mp;
 	info->uuid = g_strdup (uuid);
 	info->removable = removable;
+	info->optical = optical;
 
 	return g_node_append_data (node, info);
 }
@@ -297,17 +329,95 @@ static void
 mount_add (TrackerStorage *storage,
            const gchar    *uuid,
            const gchar    *mount_point,
-           gboolean        removable_device)
+           gboolean        removable_device,
+           gboolean        optical_disc)
 {
 	TrackerStoragePrivate *priv;
 	GNode *node;
 
 	priv = TRACKER_STORAGE_GET_PRIVATE (storage);
 
-	node = mount_add_hierarchy (priv->mounts, uuid, mount_point, removable_device);
+	node = mount_add_hierarchy (priv->mounts, uuid, mount_point, removable_device, optical_disc);
 	g_hash_table_insert (priv->mounts_by_uuid, g_strdup (uuid), node);
 
-	g_signal_emit (storage, signals[MOUNT_POINT_ADDED], 0, uuid, mount_point, NULL);
+	g_signal_emit (storage, 
+	               signals[MOUNT_POINT_ADDED], 
+	               0,
+	               uuid,
+	               mount_point,
+	               removable_device,
+	               optical_disc,
+	               NULL);
+}
+
+static gchar *
+mount_guess_content_type (GFile    *mount_root,
+                          gboolean *is_multimedia)
+{
+	gchar *content_type = NULL;
+
+	/* Set defaults */
+	*is_multimedia = FALSE;
+
+	if (g_file_has_uri_scheme (mount_root, "cdda")) {
+		*is_multimedia = TRUE;
+
+		content_type = g_strdup ("x-content/audio-cdda");
+	} else {
+		gchar **guess_type;
+		gint i;
+		
+		guess_type = g_content_type_guess_for_tree (mount_root);
+		
+		for (i = 0; guess_type && guess_type[i]; i++) {
+			if (!g_strcmp0 (guess_type[i], "x-content/image-picturecd")) {
+				/* Images */
+				content_type = g_strdup (guess_type[i]);
+				break;
+			} else if (!g_strcmp0 (guess_type[i], "x-content/video-bluray") ||
+			           !g_strcmp0 (guess_type[i], "x-content/video-dvd") ||
+			           !g_strcmp0 (guess_type[i], "x-content/video-hddvd") ||
+			           !g_strcmp0 (guess_type[i], "x-content/video-svcd") ||
+			           !g_strcmp0 (guess_type[i], "x-content/video-vcd")) {
+				/* Videos */
+				*is_multimedia = TRUE;
+				content_type = g_strdup (guess_type[i]);
+				break;
+			} else if (!g_strcmp0 (guess_type[i], "x-content/audio-cdda") ||
+			           !g_strcmp0 (guess_type[i], "x-content/audio-dvd") ||
+			           !g_strcmp0 (guess_type[i], "x-content/audio-player")) {
+				/* Audios */
+				*is_multimedia = TRUE;
+				content_type = g_strdup (guess_type[i]);
+				break;
+			} else if (!g_strcmp0 (guess_type[i], "x-content/blank-bd") ||
+			           !g_strcmp0 (guess_type[i], "x-content/blank-cd") ||
+			           !g_strcmp0 (guess_type[i], "x-content/blank-dvd") ||
+			           !g_strcmp0 (guess_type[i], "x-content/blank-hddvd")) {
+				/* Blank */
+				content_type = g_strdup (guess_type[i]);
+				break;
+			} else if (!g_strcmp0 (guess_type[i], "x-content/software") ||
+			           !g_strcmp0 (guess_type[i], "x-content/unix-software") ||
+			           !g_strcmp0 (guess_type[i], "x-content/win32-software")) {
+				/* NOTE: This one is a guess, can we
+				 * have this content type on
+				 * none-optical mount points?
+				 */
+				content_type = g_strdup (guess_type[i]);
+				break;
+			} else if (!content_type) {
+				content_type = g_strdup (guess_type[i]);
+				break;
+			}
+		}
+		
+		if (guess_type) {
+			g_strfreev (guess_type);
+		}
+	}
+
+	return content_type;
 }
 
 static void
@@ -317,8 +427,9 @@ volume_add (TrackerStorage *storage,
 {
 	TrackerStoragePrivate *priv;
 	GMount *mount;
-	gchar *str;
+	gchar *name;
 	gboolean is_mounted;
+	gboolean is_optical;
 	gchar *uuid;
 	gchar *mount_point;
 	gchar *device_file;
@@ -336,22 +447,64 @@ volume_add (TrackerStorage *storage,
 		}
 	}
 
-	str = g_volume_get_name (volume);
-	g_debug ("  Volume:'%s' found", str);
-	g_free (str);
+	name = g_volume_get_name (volume);
+	g_debug ("  Volume:'%s' found", name);
 		
 	if (!g_volume_should_automount (volume) ||
 	    !g_volume_can_mount (volume)) {
 		g_debug ("    Ignoring, volume can not be automatically mounted or mounted at all");
+		g_free (name);
 		return;
 	}
 
 	uuid = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UUID);
 	if (!uuid) {
-		g_debug ("    Ignoring, volume has no UUID");
-		return;
+		GFile *file;
+		gchar *content_type;
+		gboolean is_multimedia;
+
+		mount = g_volume_get_mount (volume); 
+		
+		if (mount) {
+			file = g_mount_get_root (mount);
+			g_object_unref (mount);
+		} else {
+			g_debug ("  Being ignored because there is no mount point and no UUID");
+			g_free (name);
+			return;
+		}
+
+		content_type = mount_guess_content_type (file, &is_multimedia);
+		g_object_unref (file);
+
+		g_debug ("  No UUID, guessed content type:'%s', has music/video:%s",
+		           content_type,
+		           is_multimedia ? "yes" : "no");
+
+		if (!is_multimedia) {
+			uuid = g_strdup (name);
+			g_debug ("  Using UUID:'%s' for optical disc", uuid);
+		}
+
+		g_free (content_type);
+
+		if (!uuid) {
+			g_debug ("  Being ignored because mount is not optical media or is music/video");
+			g_free (name);
+			return;
+		}
+
+		is_optical = TRUE;
+	} else {
+		/* We assume that all devices that are non-optical
+		 * have UUIDS already. Since optical devices are the
+		 * only ones which seem to have no UUID.
+		 */
+		is_optical = FALSE;
 	}
 
+	g_free (name);
+
 	device_file = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	g_debug ("    Device file  : %s", device_file);
 		
@@ -380,7 +533,7 @@ volume_add (TrackerStorage *storage,
 	priv = TRACKER_STORAGE_GET_PRIVATE (storage);
 
 	if (mount_point && !g_hash_table_lookup (priv->mounts_by_uuid, uuid)) {
-		mount_add (storage, uuid, mount_point, TRUE);
+		mount_add (storage, uuid, mount_point, TRUE, is_optical);
 	}
 
 	g_free (uuid);
@@ -406,6 +559,7 @@ drives_setup (TrackerStorage *storage)
 	for (ld = drives; ld; ld = ld->next) {
 		GDrive *drive;
 		GList *volumes, *lv;
+		gchar *name;
 
 		drive = ld->data;
 
@@ -414,9 +568,10 @@ drives_setup (TrackerStorage *storage)
 		}
 		
 		volumes = g_drive_get_volumes (drive);
+		name = g_drive_get_name (drive);
 
 		g_debug ("Drive:'%s' found with %d %s:",
-		         g_drive_get_name (drive),
+		         name,
 		         g_list_length (volumes),
 		         (volumes && !volumes->next) ? "volume" : "volumes");
 
@@ -427,6 +582,7 @@ drives_setup (TrackerStorage *storage)
 
 		g_list_free (volumes);
 		g_object_unref (ld->data);
+		g_free (name);
 	}
 
 	g_list_free (drives);
@@ -460,7 +616,7 @@ mount_added_cb (GVolumeMonitor *monitor,
 	if (volume) {
 		gchar *device_file;
 		gchar *uuid;
-		gboolean removable_device = TRUE;
+		gboolean removable_device;
 
 		device_file = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 		uuid = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UUID);
@@ -475,9 +631,27 @@ mount_added_cb (GVolumeMonitor *monitor,
 		/* We don't have a UUID for CDROMs */
 		if (uuid) {
 			g_message ("  Being added as a tracker resource to index!");
-			mount_add (storage, uuid, mount_point, removable_device);
+			mount_add (storage, uuid, mount_point, removable_device, FALSE);
 		} else {
-			g_message ("  Being ignored because we have no UUID");
+			gchar *content_type;
+			gboolean is_multimedia;
+
+			content_type = mount_guess_content_type (file, &is_multimedia);
+
+			g_message ("  No UUID, guessed content type:'%s', music/video:%s",
+			           content_type,
+			           is_multimedia ? "yes" : "no");
+
+			if (!is_multimedia) {
+				uuid = g_strdup (name);
+
+				g_message ("  Using UUID:'%s' for optical disc", uuid);
+				mount_add (storage, uuid, mount_point, removable_device, TRUE);
+			} else {
+				g_message ("  Being ignored because mount is not optical media or is music/video");
+			}
+
+			g_free (content_type);
 		}
 
 		g_free (uuid);
@@ -570,13 +744,17 @@ get_mount_point_by_uuid_foreach (gpointer key,
 	const gchar *uuid;
 	GNode *node;
 	MountInfo *info;
+	TrackerStorageType mount_type;
 
 	gr = user_data;
 	uuid = key;
 	node = value;
 	info = node->data;
+	mount_type = mount_info_get_type (info);
 
-	if (!gr->only_removable || info->removable) {
+	/* is mount of the type we're looking for? */
+	if ((gr->exact_match && mount_type == gr->type) ||
+	    (!gr->exact_match && ((mount_type & gr->type) == gr->type))) {
 		gchar *normalized_mount_point;
 		gint len;
 
@@ -593,19 +771,19 @@ get_mount_point_by_uuid_foreach (gpointer key,
 }
 
 /**
- * tracker_storage_get_removable_device_roots:
+ * tracker_storage_get_device_roots:
  * @storage: A #TrackerStorage
+ * @type: A #TrackerStorageType
+ * @exact_match: if all devices should exactly match the types
  *
- * Returns a #GSList of strings containing the root directories for
- * removable devices.
- *
- * Each element must be freed using g_free() and the list itself
- * through g_slist_free().
- *
- * Returns: The list of root directories.
+ * Returns: a #GSList of strings containing the root directories for
+ * devices with @type based on @exact_match. Each element must be
+ * freed using g_free() and the list itself through g_slist_free().
  **/
 GSList *
-tracker_storage_get_removable_device_roots (TrackerStorage *storage)
+tracker_storage_get_device_roots (TrackerStorage     *storage,
+				  TrackerStorageType  type,
+				  gboolean            exact_match)
 {
 	TrackerStoragePrivate *priv;
 	GetRoots gr;
@@ -615,7 +793,8 @@ tracker_storage_get_removable_device_roots (TrackerStorage *storage)
 	priv = TRACKER_STORAGE_GET_PRIVATE (storage);
 
 	gr.roots = NULL;
-	gr.only_removable = TRUE;
+	gr.type = type;
+	gr.exact_match = exact_match;
 
 	g_hash_table_foreach (priv->mounts_by_uuid,
 	                      get_mount_point_by_uuid_foreach,
@@ -625,17 +804,19 @@ tracker_storage_get_removable_device_roots (TrackerStorage *storage)
 }
 
 /**
- * tracker_storage_get_removable_device_uuids:
+ * tracker_storage_get_device_uuids:
  * @storage: A #TrackerStorage
+ * @type: A #TrackerStorageType
+ * @exact_match: if all devices should exactly match the types
  *
- * Returns a #GSList of strings containing the UUID for removable devices.
- * Each element is owned by the #GHashTable internally, the list
- * itself through should be freed using g_slist_free().
- *
- * Returns: The list of UUIDs.
+ * Returns: a #GSList of strings containing the UUID for devices with
+ * @type based on @exact_match. Each element must be freed using
+ * g_free() and the list itself through g_slist_free(). 
  **/
 GSList *
-tracker_storage_get_removable_device_uuids (TrackerStorage *storage)
+tracker_storage_get_device_uuids (TrackerStorage     *storage,
+				  TrackerStorageType  type,
+				  gboolean            exact_match)
 {
 	TrackerStoragePrivate *priv;
 	GHashTableIter iter;
@@ -654,12 +835,17 @@ tracker_storage_get_removable_device_uuids (TrackerStorage *storage)
 		const gchar *uuid;
 		GNode *node;
 		MountInfo *info;
+		TrackerStorageType mount_type;
 
 		uuid = key;
 		node = value;
 		info = node->data;
 
-		if (info->removable) {
+		mount_type = mount_info_get_type (info);
+
+		/* is mount of the type we're looking for? */
+		if ((exact_match && mount_type == type) ||
+		    (!exact_match && ((mount_type & type) == type))) {
 			uuids = g_slist_prepend (uuids, g_strdup (uuid));
 		}
 	}
